<?php

namespace Drupal\monarch_symlink\Controller;

use Drupal\Core\Url;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Render\RendererInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Drupal\Core\Controller\ControllerResolverInterface;
use Drupal\Core\Entity\Controller\EntityViewController;
use Symfony\Component\HttpKernel\Event\ControllerEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpKernel\Event\ControllerArgumentsEvent;
use Symfony\Component\EventDispatcher\LegacyEventDispatcherProxy;
use Symfony\Component\HttpKernel\Controller\ArgumentResolverInterface;

/**
 * Defines a controller to render a symlink.
 */
class SymlinkController extends EntityViewController {

  /**
   * The http kernel service.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $kernel;

  /**
   * The event dispatcher service.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $dispatcher;

  /**
   * The controller resolver service.
   *
   * @var \Drupal\Core\Controller\ControllerResolverInterface
   */
  protected $controllerResolver;

  /**
   * The the argument resolver service.
   *
   * @var \Symfony\Component\HttpKernel\Controller\ArgumentResolverInterface
   */
  protected $argumentResolver;

  /**
   * Creates a SymlinkController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Symfony\Component\HttpKernel\HttpKernelInterface $kernel
   *   The http kernel service.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $dispatcher
   *   The event dispatcher service.
   * @param \Drupal\Core\Controller\ControllerResolverInterface
   *   The controller resolver service.
   * @param \Symfony\Component\HttpKernel\Controller\ArgumentResolverInterface
   *   The the argument resolver service.
   */
  public function __construct(
      EntityTypeManagerInterface $entity_type_manager,
      RendererInterface $renderer,
      HttpKernelInterface $kernel,
      EventDispatcherInterface $dispatcher,
      ControllerResolverInterface $controller_resolver,
      ArgumentResolverInterface $argument_resolver) {

    parent::__construct($entity_type_manager, $renderer);

    $this->kernel = $kernel;
    $this->dispatcher = $dispatcher;
    $this->controllerResolver = $controller_resolver;
    $this->argumentResolver = $argument_resolver;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $dispatcher = $container->get('event_dispatcher');
    $dispatcher = LegacyEventDispatcherProxy::decorate($dispatcher);

    return new static(
      $container->get('entity_type.manager'),
      $container->get('renderer'),
      $container->get('http_kernel.basic'),
      $dispatcher,
      $container->get('controller_resolver'),
      $container->get('http_kernel.controller.argument_resolver')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function view(EntityInterface $symlink, $view_mode = 'full', $langcode = NULL) {
    $uri = $symlink->link->uri;

    try {
      $url = Url::fromUri($uri);
      $route_name = $url->getRouteName();
    }
    catch (Exception $e) {
      // @TODO Log a message if target url is invalid.
      throw new NotFoundHttpException();
    }

    return $this->getUrlResponse($url->toString());
  }

  /**
   * The _title_callback for the page that renders a symlink.
   *
   * @param \Drupal\Core\Entity\EntityInterface $symlink
   *   The current symlink.
   *
   * @return string
   *   The page title.
   */
  public function title(EntityInterface $symlink) {
    return 'Change me';
  }

  /**
   * Get the response object for a page based on a URL.
   *
   * @param string $path
   *   The URL path of the page to fetch.
   *
   * @return mixed The response from the path controller.
   */
  protected function getUrlResponse(string $path) {
    // Create the sub request.
    $sub_request = Request::create($path, 'GET');
    $type = HttpKernelInterface::SUB_REQUEST;
    $event = new RequestEvent($this->kernel, $sub_request, $type);
    $this->dispatcher->dispatch($event, KernelEvents::REQUEST);

    // Load the request controller.
    $controller = $this->controllerResolver->getController($sub_request);
    $event = new ControllerEvent($this->kernel, $controller, $sub_request, $type);
    $this->dispatcher->dispatch($event, KernelEvents::CONTROLLER);
    $controller = $event->getController();

    // Load the controller arguments.
    $arguments = $this->argumentResolver->getArguments($sub_request, $controller);
    $event = new ControllerArgumentsEvent($this->kernel, $controller, $arguments, $sub_request, $type);
    $this->dispatcher->dispatch($event, KernelEvents::CONTROLLER_ARGUMENTS);
    $controller = $event->getController();
    $arguments = $event->getArguments();

    // Call the controller.
    return $controller(...$arguments);
  }

}
